from Globals import InitializeClass
from Products.Archetypes.Widget import StringWidget
from Products.Archetypes.Registry import registerWidget,registerPropertyType
from AccessControl import ClassSecurityInfo
from config import OUTPUT_FORMAT

class ColorPickerWidget(StringWidget):
    _properties = StringWidget._properties.copy()
    _properties.update(
        {'macro' : 'colorpicker',
         'helper_js': ('colorpicker.js',),
	 'allow_brightness':True,
	 'output_format':OUTPUT_FORMAT,
        })

    security = ClassSecurityInfo()
    
    def convertColor(self,colorstring):
        """ Convert color to a given color format """
    
        if self.output_format == 'HexColor':
           return self.HTMLColorToHexColor(colorstring)
        elif self.output_format == 'RGB':
           return self.HTMLColorToRGB(colorstring)
        elif self.output_format == 'PilColor':
           return self.HTMLColorToPILColor(colorstring)

        return colorstring

    # color converting methods taken from:
    # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/266466

    def HTMLColorToHexColor(self,colorstring):
        """ convert an (R, G, B) tuple to #RRGGBB """

        colorstring = colorstring.strip()
	if colorstring[:2] == '0x': return colorstring
	#raise str(colorstring[:2])
        if colorstring[0] == '#': colorstring = colorstring[1:]
        if len(colorstring) != 6:
            raise ValueError, "input #%s is not in #RRGGBB format" % colorstring
        hexcolor = '0x%s' % colorstring 
        return hexcolor

    def HTMLColorToRGB(self,colorstring):
        """ convert #RRGGBB to an (R, G, B) tuple """
        colorstring = colorstring.strip()
        if colorstring[0] == '#': colorstring = colorstring[1:]
        if len(colorstring) != 6:
            raise ValueError, "input #%s is not in #RRGGBB format" % colorstring
        r, g, b = colorstring[:2], colorstring[2:4], colorstring[4:]
        r, g, b = [int(n, 16) for n in (r, g, b)]
        return (r, g, b)

    def HTMLColorToPILColor(self,colorstring):
        """ converts #RRGGBB to PIL-compatible integers"""
        colorstring = colorstring.strip()
        while colorstring[0] == '#': colorstring = colorstring[1:]
        # get bytes in reverse order to deal with PIL quirk
        colorstring = colorstring[-2:] + colorstring[2:4] + colorstring[:2]
        # finally, make it numeric
        color = int(colorstring, 16)
        return color


registerWidget(ColorPickerWidget,
               title='Color',
               description=('Pick a color!'),
               used_for=('Products.Archetypes.public.StringField',)
               )

registerPropertyType('allow_brightness', 'boolean', ColorPickerWidget)
registerPropertyType('output_format', 'string', ColorPickerWidget)
